/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.cache;

import java.util.logging.Logger;
import java.util.logging.Level;
import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.util.ArrayList;
import javax.sql.DataSource;
import javax.naming.NamingException;
import javax.naming.InitialContext;

import com.funambol.framework.tools.DBTools;

import com.funambol.email.util.Def;

/**
 * Contains methods to access to the table with the state of the items.
 *
 * @version $Id: CrcItemStateDAO.java,v 1.2 2006/02/06 08:58:51 gbmiglia Exp $
 *
 */
public class CrcItemStateDAO {

    // --------------------------------------------------------------- Constants

    private static final String SQL_SELECT_ITEMS_BY_PRINCIPAL_BY_SOURCE    =
        "select id, last_crc " +
        "from fnbl_email_cache where source_uri = ? and principal = ?";

    private static final String SQL_ADD_LOCAL_ITEM                         =
        "insert into fnbl_email_cache " +
        "(id, last_crc, source_uri, principal) values(?, ?, ?, ?)"                                                   ;

    private static final String SQL_DELETE_LOCAL_ITEMS                     =
        "delete from fnbl_email_cache where source_uri = ? and principal = ? ";


    private Logger log = Logger.getLogger(Def.LOGGER_NAME);

    // ------------------------------------------------------------ Private data

    /**         */
    private String jndiDataSourceName;

    /**         */
    private DataSource dataSource;


    // ------------------------------------------------------------- Constructor

    /**
     *
     */
    private CrcItemStateDAO() {
    }

    /**
     *
     *
     * @param jndiDataSourceName String
     */
    public CrcItemStateDAO(String jndiDataSourceName) {
        this.jndiDataSourceName = jndiDataSourceName;
    }


    // ---------------------------------------------------------- Public Methods

    /**
     *
     * @throws Exception
     */
    public void init() throws Exception {
        lookupDataSource();
    }

    /**
     *
     *
     * @return String
     */
    public String getJndiDataSourceName() {
        return jndiDataSourceName;
    }

    /**
     *
     * @param jndiDataSourceName String
     */
    public void setJndiDataSourceName(String jndiDataSourceName) {
        this.jndiDataSourceName = jndiDataSourceName;
    }

    /**
     * Adds local items
     *
     *
     * @param items CrcSyncItemInfo[]
     * @param sourceURI String
     * @param principal String
     * @throws Exception
     */
    public void addLocalItems(CrcSyncItemInfo[] items,
                              String sourceURI,
                              String principal) throws Exception {

         Connection         connection    = null ;
         PreparedStatement  ps            = null ;

         String             id            = null;
         long               lastCrc       ;

         try {

            connection = dataSource.getConnection();

            if (log.isLoggable(Level.FINEST)) {
               log.finest("CrcItemStateDAO add start " + items.length);
            }

            for (int i=0, l = items.length; i < l; i++) {

                ps = connection.prepareStatement(SQL_ADD_LOCAL_ITEM);

                id       = items[i].getKey().getKeyAsString();
                lastCrc  = items[i].getLastCrc() ;

                if (log.isLoggable(Level.FINEST)) {
                   log.finest("   CrcItemStateDAO add " + id + " - " + lastCrc);
                }
                ps.setString (1, id        ) ;
                ps.setLong   (2, lastCrc   ) ;
                ps.setString (3, sourceURI ) ;
                ps.setString (4, principal ) ;

                ps.executeUpdate();

                ps.close() ;
                ps = null  ;
            }
         } catch (Exception e) {
             throw new Exception("Error adding local items", e);
         } finally {
             DBTools.close(connection, ps, null);
         }
    }

    /**
     * Delete local items
     *
     *
     * @param sourceURI String
     * @param principal String
     * @throws Exception
     */
    public void deleteLocalItems(String sourceURI, String principal)
        throws Exception {

        Connection         connection  = null ;
        PreparedStatement  ps          = null ;

        if (log.isLoggable(Level.FINEST)) {
            log.finest("CrcItemStateDAO delete start "+ principal+ " - " +sourceURI);
        }
        try {

            connection = dataSource.getConnection();

            ps = connection.prepareStatement(SQL_DELETE_LOCAL_ITEMS);

            ps.setString  (1, sourceURI ) ;
            ps.setString  (2, principal ) ;

            ps.executeUpdate();

        } catch (Exception e) {
            throw new Exception("Error deleting local items", e);
        } finally {
            DBTools.close(connection, ps, null);
        }
    }

    /**
     * Update local items
     *
     *
     * @param localItems CrcSyncItemInfo[]
     * @param sourceURI String
     * @param principal String
     * @throws Exception
     */
    public void updateLocalItems(CrcSyncItemInfo[] localItems ,
                                 String sourceURI  ,
                                 String principal  )
    throws Exception {

         if (log.isLoggable(Level.FINEST)) {
            log.finest("CrcItemStateDAO update start");
         }
         this.deleteLocalItems (sourceURI, principal);
         this.addLocalItems    (localItems, sourceURI, principal);
         if (log.isLoggable(Level.FINEST)) {
            log.finest("CrcItemStateDAO update end");
         }
    }

    /**
     * Gets local items
     *
     *
     * @param sourceURI String
     * @param principalId String
     * @return CrcSyncItemInfo[]
     * @throws Exception
     */
    public CrcSyncItemInfo[] getLocalItems (String sourceURI,
                                            String principalId)
        throws Exception {

         Connection connection   = null;
         PreparedStatement ps    = null;
         ResultSet rs            = null;
         ArrayList rows          = null;
         CrcSyncItemInfo[] items = null;

         try {

             connection = dataSource.getConnection();

             rows = new ArrayList();

             ps = connection.prepareStatement(
                    SQL_SELECT_ITEMS_BY_PRINCIPAL_BY_SOURCE);

             ps.setString(1, sourceURI);
             ps.setString(2, principalId);

             rs = ps.executeQuery();

             while(rs.next()) {
                 rows.add(new CrcSyncItemInfo(rs.getString(1), rs.getLong(2)));
             }

             items = (CrcSyncItemInfo[]) rows.toArray(new CrcSyncItemInfo[0]);

         } catch (Exception e) {
             throw new Exception("Error getting local items", e);
         } finally {
             DBTools.close(connection, ps, rs);
         }

         return items;
    }



    // --------------------------------------------------------- Private Methods

    /**
     * Lookups the datasource
     *
     * JBOSS
     * private String jndiDataSourceName = "java:jdbc/sync4j";
     *
     * TOMCAT
     * private String jndiDataSourceName = "java:comp/env/jdbc/sync4j";
     *
     * @throws Exception if a error occurs during lookup
     */
    private void lookupDataSource() throws Exception {

        InitialContext ctx = new InitialContext();
        if (log.isLoggable(Level.FINEST)) {
              log.finest("CachedSyncSource: datasource lookup");
        }

        try {
          // try tomcat  i.e. java:comp/env/jdbc/sync4j
          dataSource = (DataSource) ctx.lookup("java:comp/env/"+jndiDataSourceName) ;
          if (log.isLoggable(Level.FINEST)) {
             log.finest("CachedSyncSource: datasource lookup done using java:comp/env/ prefix");
          }
        }
        catch (NamingException ne) {
          if (log.isLoggable(Level.FINEST)) {
              log.finest("CachedSyncSource: No tomcat datasource lookup!");
          }
          try {
              // try jboss i.e. java:jdbc/sync4j
              dataSource = (DataSource) ctx.lookup("java:"+jndiDataSourceName) ;
              if (log.isLoggable(Level.FINEST)) {
                 log.finest("CachedSyncSource: datasource lookup done using java: prefix");
              }
          }
          catch (Exception ex){
            if (log.isLoggable(Level.FINEST)) {
                log.finest("CachedSyncSource: No jboss datasource lookup!");
            }
            try {
                // try othser j2ee server jdbc/sync4j
                dataSource = (DataSource) ctx.lookup(""+jndiDataSourceName) ;
                if (log.isLoggable(Level.FINEST)) {
                   log.finest("CachedSyncSource: datasource lookup done using - prefix");
                }
            }
            catch (Exception e){
              if (log.isLoggable(Level.FINEST)) {
                log.finest("CachedSyncSource: No j2ee server datasource lookup!");
              }
              throw new Exception("Data source "+ jndiDataSourceName
                                  + " not found " + e.toString());
            }
          }
        }
    }

}
