/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.security ;


import java.util.logging.Logger;
import java.util.logging.Level;
import java.security.Principal;

import com.funambol.framework.security.Officer;
import com.funambol.framework.core.Cred;
import com.funambol.framework.core.Authentication;
import com.funambol.framework.security.Sync4jPrincipal;
import com.funambol.framework.tools.Base64;
import com.funambol.framework.server.store.PersistentStore;
import com.funambol.framework.server.store.PersistentStoreException;
import com.funambol.framework.server.store.NotFoundException;
import com.funambol.server.config.Configuration;

import com.funambol.email.transport.MailServerWrapperFactory;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.util.Def;
import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.exception.EmailConfigException;
import com.funambol.email.admin.EmailConnectorConfig;


/**
 * This is an implementation of the <i>Officier</i> interface.
 * It always authenticates and authorizes users and resource accesses.
 *
 * @version $Id: EmailOfficer.java,v 1.3 2006/02/06 09:03:23 gbmiglia Exp $
 */
public class EmailOfficer implements Officer, java.io.Serializable {

    // ---------------------------------------------------------- Protected data

    protected Logger  log = Logger.getLogger(Def.LOGGER_NAME);

    protected PersistentStore ps = null;

    // ------------------------------------------------------------- Public data

    // -------------------------------------------------------------- Properties

    /**
     * Has the last login failed for incorrect login/password?
     */
    private boolean loginFailed = false;

    /**
     *
     * @return boolean
     */
    public boolean isLoginFailed() {
        return loginFailed;
    }

    /**
     * Has the last login failed for expired temporary login?
     */
    private boolean loginExpired = false;

    /**
     * Which type of authetication does impose the server?
     */
    private String clientAuth = Cred.AUTH_TYPE_BASIC;

    /**
     *
     *
     * @return String
     */
    public String getClientAuth() {
        return this.clientAuth;
    }

    /**
     *
     * @param clientAuth String
     */
    public void setClientAuth(String clientAuth) {
        this.clientAuth = clientAuth;
    }

    /**
     * Which type of authetication use the server to send his credential?
     */
    private String serverAuth = Cred.AUTH_NONE;

    /**
     *
     *
     * @return String
     */
    public String getServerAuth() {
        return this.serverAuth;
    }


    /**
     *
     *
     * @param serverAuth String
     */
    public void setServerAuth(String serverAuth) {
        this.serverAuth = serverAuth;
    }

    /**
     * SSL activation
     */
    private boolean isSSL = false;

    /**
     *
     * @return boolean
     */
    public boolean getIsSSL() {
        return this.isSSL;
    }

    /**
     *
     *
     * @param isSSL boolean
     */
    public void setIsSSL(boolean isSSL) {
        this.isSSL = isSSL;
    }


    // ---------------------------------------------------------- Public methods

    /**
     * Authenticates a credential.
     *
     * @param credential the credential to be authenticated
     *
     * @return true if the credential is autenticated, false otherwise
     */
    public boolean authenticate(Cred credential) {
        Configuration config = Configuration.getConfiguration();
        ps = config.getStore();

        String type = credential.getType();

        if ((Cred.AUTH_TYPE_BASIC).equals(type)) {
            return authenticateBasicCredential(credential);
        }
        return false;
    }

    /**
     * Authorizes a resource.
     *
     * @param principal the requesting entity
     * @param resource the name (or the identifier) of the resource to be authorized
     *
     * @return true if the credential is authorized to access the resource, false
     *         otherwise
     */
    public boolean authorize(Principal principal, String resource) {
        return true;
    }

    /**
     * Un-authenticates a credential.
     * Do nothing. In the current implementation, the authentication is
     * discarde as soon as the LoginContext is garbage collected.
     *
     * @param credential the credential to be unauthenticated
     */
    public void unAuthenticate(Cred credential) {
    }

    /**
     *
     * @return boolean
     */
    public boolean isAccountExpired() {
        return loginExpired;
    }


    // --------------------------------------------------------- Private methods

    /**
     *
     *
     * @param credential Cred
     * @return boolean
     */
    private boolean authenticateBasicCredential(Cred credential) {

        String username = null, password = null;
        Sync4jPrincipal principal = null;

        Authentication auth = credential.getAuthentication();
        String deviceId     = auth.getDeviceId();
        String userpwd      = new String(Base64.decode(auth.getData()));

        int p = userpwd.indexOf(':');

        if (p == -1) {
            username = userpwd;
            password = "";
        } else {
            username = (p>0) ? userpwd.substring(0, p) : "";
            password = (p == (userpwd.length()-1)) ? "" : userpwd.substring(p+1);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Username: " + username);
        }

        //
        // Verify that exist the principal for the
        // specify deviceId and username
        //
        boolean principalFound = false;
        try {
            principal = new Sync4jPrincipal(username, deviceId);
            ps.read(principal);
            credential.getAuthentication().setPrincipalId(principal.getId());
            principalFound = true;
        } catch (NotFoundException nfe) {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("Principal for " + username + ":" + deviceId + " not found");
            }
        } catch(PersistentStoreException e) {
            log.severe("Error reading principal: " + e);
            log.throwing(getClass().getName(), "authenticateBasicCredential", e);
            return false;
        }

        //
        // Verify these credentials on Mail Server
        //
        boolean isMailUser = checkMailCredentials(username, password);

        if (!isMailUser) {
            //
            // User not authenticate
            //
            return false;
        }

        if (!principalFound) {
            //
            // We must create a new principal
            //
            principal = new Sync4jPrincipal(username, deviceId);
            try {
                ps.store(principal);
            } catch (PersistentStoreException ex) {
                log.severe("Error creating new principal: " + ex);
                log.throwing(getClass().getName(), "authenticateBasicCredential", ex);
                return false;
            }

            credential.getAuthentication().setPrincipalId(principal.getId());
        }

        return true;
    }

    /**
     * Checks if the given login and password are valid
     * credentials on Mail Server
     *
     *
     * @param login String
     * @param password String
     * @return boolean
     */
    private boolean checkMailCredentials(String login,
                                         String password) {


        String inHost   = "";
        String outHost  = "";
        String protocol = "pop3";
        int    port     = 110;
        try {
          inHost   = EmailConnectorConfig.getConfigInstance().getInHost();
          outHost  = EmailConnectorConfig.getConfigInstance().getOutHost();
          protocol = EmailConnectorConfig.getConfigInstance().getProtocol();
          port     = EmailConnectorConfig.getConfigInstance().getPort();
        } catch (EmailConfigException e) {
            log.severe("Error Getting Connector Parameters:: " + e);
            log.throwing(getClass().getName(), "checkMailCredentials", e);
        }

        IMailServerWrapper mswf =
                  MailServerWrapperFactory.getMailServerWrapper(protocol);

        boolean isconnected = false;
        boolean error       = false;
        try {
            mswf.checkUser(outHost,
                           inHost,
                           ""+port,
                           login,
                           password,
                           this.isSSL);

        } catch (EmailAccessException dae){
            log.severe("Error getting connection: " + dae);
            log.throwing(getClass().getName(), "checkMailCredentials", dae);
            error = true;
        }

        if (!error){
           isconnected = true;
        }

        mswf = null;

        return isconnected;
    }

}
